﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Vintagestory.API.Client;
using Vintagestory.API.Server;
using Vintagestory.API.Common;
using Vintagestory.GameContent;
using Vintagestory.API.Config;
using Vintagestory.API.MathTools;
using Vintagestory.API.Common.Entities;
using ProtoBuf;

namespace vsmovemod.src
{
    class BehaviorClimbing
    {
        public static void DisableClimbingDown(IClientPlayer player, ICoreClientAPI api)
        {
            Block aboveBlock = api.World.BlockAccessor.GetBlock((int)player.Entity.Pos.X, (int)player.Entity.Pos.Y + 1, (int)player.Entity.Pos.Z);
            Block aboveBlock2 = api.World.BlockAccessor.GetBlock((int)player.Entity.Pos.X, (int)player.Entity.Pos.Y + 2, (int)player.Entity.Pos.Z);

            // Prevent player from climbing down (bouldering)
            if (player.Entity.Controls.IsClimbing && !DataFields.isClimbable && !aboveBlock.Climbable && !aboveBlock2.Climbable && !DataFields.switchedClimbingControls)
            {
                player.Entity.Controls.Sneak = false;
                api.Input.InWorldAction += MoveMod.Input_InWorldAction3;
                DataFields.switchedClimbingControls = true;
            }
            else if (!DataFields.isCrawling && !DataFields.inBoatMode)
            {
                api.Input.InWorldAction += MoveMod.Input_InWorldAction1;
                DataFields.switchedClimbingControls = false;
            }
        }

        public static void EnableClimbing(IClientPlayer player, ICoreClientAPI api)
        {
            Block aboveBlock = api.World.BlockAccessor.GetBlock((int)player.Entity.Pos.X, (int)player.Entity.Pos.Y + 1, (int)player.Entity.Pos.Z);
            Block aboveBlock2 = api.World.BlockAccessor.GetBlock((int)player.Entity.Pos.X, (int)player.Entity.Pos.Y + 2, (int)player.Entity.Pos.Z);

            // Ability to climb
            if (!player.Entity.Controls.IsClimbing)
            {
                if (player.Entity.Controls.Sneak && DataFields.stamina > DataFields.drainStaminaClimbing && player.InventoryManager.ActiveHotbarSlot.Empty)
                {
                    player.Entity.Properties.CanClimbAnywhere = true;
                }
                else
                {
                    player.Entity.Properties.CanClimbAnywhere = false;
                    DataFields.letGoClimbing = false;
                }
            }
            else
            {
                if (DataFields.stamina > 0 && player.InventoryManager.ActiveHotbarSlot.Empty && !DataFields.letGoClimbing)
                {
                    player.Entity.Properties.CanClimbAnywhere = true;
                }
                else if (DataFields.isClimbable == true)
                {
                    player.Entity.Properties.CanClimbAnywhere = true;
                }
                else
                {
                    Block BlockatPlayerZMinus = api.World.BlockAccessor.GetBlock((int)player.Entity.Pos.X, (int)player.Entity.Pos.Y, (int)player.Entity.Pos.Z - 1);
                    Block BlockatPlayerZPlus = api.World.BlockAccessor.GetBlock((int)player.Entity.Pos.X, (int)player.Entity.Pos.Y, (int)player.Entity.Pos.Z + 1);

                    // Sound loose grip (no stamina climbing)
                    if (!DataFields.isClimbable && !aboveBlock.Climbable && !BlockatPlayerZMinus.Climbable && !BlockatPlayerZPlus.Climbable)
                    {
                        api.World.PlaySoundFor(new AssetLocation("vsmovemod:sounds/loosegrip"), player, false, 35, 0.2f);
                    }
                    player.Entity.Properties.CanClimbAnywhere = false;
                    DataFields.letGoClimbing = false;
                }
            }

            // Fix no ladderup animation when holding sprint
            if (player.Entity.Controls.IsClimbing && (player.Entity.Controls.TriesToMove || player.Entity.Controls.Jump) && !player.Entity.Controls.Sneak)
            {
                player.Entity.StartAnimation("climbup");
            }
            else if (player.Entity.Controls.IsClimbing && !(player.Entity.Controls.TriesToMove || player.Entity.Controls.Jump) && !player.Entity.Controls.Sneak)
            {
                player.Entity.StopAnimation("climbup");
            }
            else if (!player.Entity.Controls.IsClimbing && !player.Entity.Controls.Sneak)
            {
                player.Entity.StopAnimation("climbup");
            }

            // Stop crouch animation while climbing (Crawl-climb fix)
            if (player.Entity.Controls.IsClimbing)
            {
                player.Entity.StopAnimation("gleiten");
                player.Entity.StopAnimation("liegen");
            }

            // Stamina drain & Sound-FX while climbing
            if (player.Entity.Controls.IsClimbing && DataFields.stamina > 0 && (player.Entity.Controls.TriesToMove || player.Entity.Controls.Sneak || player.Entity.Controls.Jump))
            {
                // Sound random climbing
                if (DataFields.soundClimbinglCountdown < 10)
                {
                    Random rnd = new Random();
                    api.World.PlaySoundFor(new AssetLocation("vsmovemod:sounds/climb"), player, false, 5, 0.2f);
                    //Playback-rate randomizer
                    DataFields.soundClimbinglCountdown = rnd.Next(30, 60);
                }

                float staminaNewValue = (DataFields.stamina -= DataFields.drainStaminaClimbing);
                if (DataFields.isClimbable == false)
                {
                    if (staminaNewValue >= 0)
                    {
                        DataFields.stamina = staminaNewValue;
                    }
                    else
                    {
                        DataFields.stamina = 0;
                    }
                }
                else if (DataFields.isClimbable == true)
                {
                    DataFields.stamina += DataFields.drainStaminaClimbing;
                }
            }
            else if (player.Entity.Controls.IsClimbing && DataFields.stamina > 0 && !(player.Entity.Controls.TriesToMove || player.Entity.Controls.Sneak || player.Entity.Controls.Jump))
            {
                float staminaNewValue = (DataFields.stamina -= (DataFields.drainStaminaClimbing * DataFields.drainStaminaIdleModificator));
                if (DataFields.isClimbable == false)
                {
                    if (staminaNewValue >= 0)
                    {
                        DataFields.stamina = staminaNewValue;
                    }
                    else
                    {
                        DataFields.stamina = 0;
                    }
                }
                else if (DataFields.isClimbable == true)
                {
                    DataFields.stamina += (DataFields.drainStaminaClimbing * DataFields.drainStaminaIdleModificator);
                }
            }
        }
    }
}
