﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Vintagestory.API.Client;
using Vintagestory.API.Server;
using Vintagestory.API.Common;
using Vintagestory.GameContent;
using Vintagestory.API.Config;
using Vintagestory.API.MathTools;
using Vintagestory.API.Common.Entities;
using ProtoBuf;

namespace vsmovemod.src
{
    class BehaviorDash
    {
        public static void DashStop(IClientPlayer player, ICoreClientAPI api)
        {
            // Stop Motion Vec AfterDash
            if (DataFields.dashTime == 5 && DataFields.motionStopedAfterDash == false && player.Entity.OnGround)
            {
                Vec3d motionZero = new Vec3d(0, 0, 0);
                player.Entity.Pos.Motion = motionZero;
                DataFields.motionStopedAfterDash = true;
            }
        }

        public static void DashMovement(IClientPlayer player, ICoreClientAPI api) 
        {
            // Dash start & animation
            if ((player.Entity.Controls.Sprint || player.Entity.Controls.Jump) && player.Entity.Controls.TriesToMove && player.Entity.Controls.RightMouseDown && !DataFields.hasGlided)
            {
                //Dash Forward
                if (DataFields.stamina > DataFields.drainStaminaDash && DataFields.dashCountdown < 1 && !player.Entity.Controls.Backward && !player.Entity.Controls.Right && !player.Entity.Controls.Left)
                {
                    player.Entity.StartAnimation("dash");
                    DataFields.stamina = DataFields.stamina -= DataFields.drainStaminaDash;
                    DataFields.dashCountdown = DataFields.maxDashCountdown;
                    DataFields.staminaRegenDelay = DataFields.maxStaminaRegenDelay;
                    DataFields.dashTime = 0;
                    DataFields.motionStopedAfterDash = false;

                    Random rnd = new Random();
                    int rndInt;
                    rndInt = rnd.Next(1, 3);
                    if (rndInt == 1)
                    {
                        api.World.PlaySoundFor(new AssetLocation("vsmovemod:sounds/dashforward-0"), player, false, 1.1f, 0.2f);
                    }
                    else if (rndInt == 2)
                    {
                        api.World.PlaySoundFor(new AssetLocation("vsmovemod:sounds/dashforward-1"), player, false, 1.1f, 0.2f);
                    }
                    else if (rndInt == 3)
                    {
                        api.World.PlaySoundFor(new AssetLocation("vsmovemod:sounds/dashforward-2"), player, false, 1.1f, 0.2f);
                    }
                }

                //Dash Right
                else if (DataFields.stamina > DataFields.drainStaminaDash && DataFields.dashCountdown < 1 && !player.Entity.Controls.Backward && player.Entity.Controls.Right)
                {
                    player.Entity.StartAnimation("dashright");
                    DataFields.stamina = DataFields.stamina -= DataFields.drainStaminaDash;
                    DataFields.dashCountdown = DataFields.maxDashCountdown;
                    DataFields.staminaRegenDelay = DataFields.maxStaminaRegenDelay;
                    DataFields.dashTime = 0;
                    DataFields.motionStopedAfterDash = false;

                    api.World.PlaySoundFor(new AssetLocation("vsmovemod:sounds/dash"), player, false, 2, 0.3f);
                }

                // Dash Left
                else if (DataFields.stamina > DataFields.drainStaminaDash && DataFields.dashCountdown < 1 && !player.Entity.Controls.Backward && player.Entity.Controls.Left)
                {
                    player.Entity.StartAnimation("dashleft");
                    DataFields.stamina = DataFields.stamina -= DataFields.drainStaminaDash;
                    DataFields.dashCountdown = DataFields.maxDashCountdown;
                    DataFields.staminaRegenDelay = DataFields.maxStaminaRegenDelay;
                    DataFields.dashTime = 0;
                    DataFields.motionStopedAfterDash = false;

                    api.World.PlaySoundFor(new AssetLocation("vsmovemod:sounds/dash"), player, false, 2, 0.3f);
                }

                // Backflip
                else if (DataFields.stamina > DataFields.drainStaminaDash && DataFields.dashCountdown < 1 && player.Entity.Controls.Backward)
                {
                    player.Entity.StartAnimation("backflip");
                    DataFields.stamina = DataFields.stamina -= DataFields.drainStaminaDash;
                    DataFields.dashCountdown = DataFields.maxDashCountdown;
                    DataFields.staminaRegenDelay = DataFields.maxStaminaRegenDelay;
                    DataFields.dashTime = 0;
                    DataFields.motionStopedAfterDash = false;

                    api.World.PlaySoundFor(new AssetLocation("vsmovemod:sounds/backflip"), player, false, 4, 0.2f);

                }

                // Not enough Stamina
                else if (DataFields.stamina < DataFields.drainStaminaDash)
                {
                    DataFields.stamina = 0;
                    DataFields.staminaRegenDelay = DataFields.maxStaminaRegenDelay;
                }
            }

            // Dash-Movement /  50ms
            if (DataFields.dashTime < 5)
            {
                if (!player.Entity.CollidedVertically && !player.Entity.Controls.Backward)
                {
                    player.Entity.Pos.Motion = player.Entity.Pos.Motion * 1.10;
                    DataFields.dashTime++;
                }
                else if (!player.Entity.CollidedVertically && player.Entity.Controls.Backward)
                {
                    player.Entity.Pos.Motion = player.Entity.Pos.Motion * 1;
                    player.Entity.Pos.Motion.Y = player.Entity.Pos.Motion.Y * 0.6;
                    DataFields.dashTime++;
                }
                else
                {
                    player.Entity.Pos.Motion = player.Entity.Pos.Motion * 2;
                    DataFields.dashTime++;
                }
            }

            // Dash anticipation / better running Animation
            if ((player.Entity.CurrentControls == ((EnumEntityActivity)4 | (EnumEntityActivity)2)) && player.Entity.Controls.TriesToMove && DataFields.dashCountdown < 1 && DataFields.stamina > DataFields.drainStaminaDash)
            {
                player.Entity.StartAnimation("awaitdash");
            }
            else
            {
                player.Entity.StopAnimation("awaitdash");
            }

            // Reset dash animation
            if (DataFields.dashCountdown <= DataFields.maxDashCountdown - 6)
            {
                player.Entity.StopAnimation("dash");
                player.Entity.StopAnimation("dashright");
                player.Entity.StopAnimation("dashleft");
            }
        }
    }
}
