﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Vintagestory.API;
using Vintagestory.API.Datastructures;
using Vintagestory.API.Client;
using Vintagestory.API.Server;
using Vintagestory.API.Common;
using Vintagestory.GameContent;
using Vintagestory.API.Config;
using Vintagestory.API.MathTools;
using Vintagestory.API.Common.Entities;
using ProtoBuf;

namespace vsmovemod.src
{
    class BlockSailboat : Block
    {
        public override bool TryPlaceBlock(IWorldAccessor world, IPlayer byPlayer, ItemStack itemstack, BlockSelection blockSel, ref string failureCode)
        {
            if (AllowSnowCoverage(world, blockSel.Position.UpCopy()))
            {
                if (!world.Claims.TryAccess(byPlayer, blockSel.Position, EnumBlockAccessFlags.BuildOrBreak))
                {
                    byPlayer.InventoryManager.ActiveHotbarSlot.MarkDirty();
                    return false;
                }

                blockSel = blockSel.Clone();
                blockSel.Position = blockSel.Position.Up();

                BlockFacing[] horVer = SuggestedHVOrientation(byPlayer, blockSel);
                BlockPos secondPos = blockSel.Position.AddCopy(horVer[0]);

                BlockSelection secondBlockSel = new BlockSelection() { Position = secondPos, Face = BlockFacing.UP };
                if (!CanPlaceBlock(world, byPlayer, secondBlockSel, ref failureCode)) return false;

                string code = horVer[0].Code;

                Block orientedBlock = world.BlockAccessor.GetBlock(CodeWithParts("hull", code));
                orientedBlock.DoPlaceBlock(world, byPlayer, secondBlockSel, itemstack);

                AssetLocation hullCode = CodeWithParts("rear", code);
                orientedBlock = world.BlockAccessor.GetBlock(hullCode);
                orientedBlock.DoPlaceBlock(world, byPlayer, blockSel, itemstack);
                return true;
            }

            return false;
        }

        public override void OnBlockRemoved(IWorldAccessor world, BlockPos pos)
        {
            string boatpart = LastCodePart(1);
            BlockFacing facing = BlockFacing.FromCode(LastCodePart());

            if (LastCodePart(1) == "rear")
            {
            }
            else if (LastCodePart(1) == "hull")
            {
                facing = BlockFacing.FromCode(LastCodePart());
                facing = facing.Opposite;
            }

            BlockPos deletePos = pos.AddCopy(facing);
            Block secondBlock = world.BlockAccessor.GetBlock(deletePos);

            if (secondBlock is BlockSailboat)
            {
                world.BlockAccessor.SetBlock(0, deletePos);
            }

            base.OnBlockRemoved(world, pos);
        }

        public override bool AllowSnowCoverage(IWorldAccessor world, BlockPos pos)
        {
            Block block1 = world.BlockAccessor.GetBlock(pos.DownCopy());

            Block block2 = world.BlockAccessor.GetBlock(pos.NorthCopy().DownCopy());
            Block block3 = world.BlockAccessor.GetBlock(pos.EastCopy().DownCopy());
            Block block4 = world.BlockAccessor.GetBlock(pos.SouthCopy().DownCopy());
            Block block5 = world.BlockAccessor.GetBlock(pos.WestCopy().DownCopy());

            Block block6 = world.BlockAccessor.GetBlock(pos.NorthCopy());
            Block block7 = world.BlockAccessor.GetBlock(pos.EastCopy());
            Block block8 = world.BlockAccessor.GetBlock(pos.SouthCopy());
            Block block9 = world.BlockAccessor.GetBlock(pos.WestCopy());

            return

                   block1.IsLiquid() && block1.LiquidLevel == 7 && block1.LiquidCode.Contains("water") &&

                   block2.IsLiquid() && block2.LiquidLevel == 7 && block2.LiquidCode.Contains("water") &&
                   block3.IsLiquid() && block3.LiquidLevel == 7 && block3.LiquidCode.Contains("water") &&
                   block4.IsLiquid() && block4.LiquidLevel == 7 && block4.LiquidCode.Contains("water") &&
                   block5.IsLiquid() && block5.LiquidLevel == 7 && block5.LiquidCode.Contains("water") &&

                   !block6.IsLiquid() && block6.Code.Path.StartsWith("air") &&
                   !block7.IsLiquid() && block7.Code.Path.StartsWith("air") &&
                   !block8.IsLiquid() && block8.Code.Path.StartsWith("air") &&
                   !block9.IsLiquid() && block9.Code.Path.StartsWith("air");

                   
        }
    }
}
