﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Vintagestory.API.Client;
using Vintagestory.API.Server;
using Vintagestory.API.Common;
using Vintagestory.GameContent;
using Vintagestory.API.Config;
using Vintagestory.API.MathTools;
using Vintagestory.API.Common.Entities;
using ProtoBuf;

namespace vsmovemod.src
{
    class ItemGlider : ItemWearable
    {
        public static void StopAnimations(IClientPlayer player)
        {
            // Stop Animation when inGliderMode
            if (DataFields.inGliderMode && DataFields.hasGlided)
            {
                player.Entity.StopAnimation("walk");
                player.Entity.StopAnimation("idle");
                player.Entity.StopAnimation("awaitdash");
                player.Entity.StopAnimation("land");
                player.Entity.StopAnimation("glide");
                if (DataFields.gliderDelay == 150)
                {
                    // stop animation sprint a few moments after take off
                    player.Entity.StopAnimation("sprint");
                }
            }
        }
        public static void UseGlider(IClientPlayer player, ICoreClientAPI api)
        {
            //Check for glider wearable in slot
            var gearInv = player.Entity.GearInventory;

            foreach (var slot in gearInv)
            {
                ItemWearable item;
                if (slot.Empty || (item = slot.Itemstack.Collectible as ItemWearable) == null) continue; // null Exception catch

                if (slot.Itemstack.Collectible.Code.Path.StartsWith("glider-body-"))
                {
                    DataFields.wearingGliderT1 = true;

                    if (slot.Itemstack.Collectible.Code.Path.StartsWith("glider-body-crafted-unfolded-rainbow") ||
                        slot.Itemstack.Collectible.Code.Path.StartsWith("glider-body-antique-unfolded-rainbow"))
                    {
                        Particles.SpawnSpecialParticles(player.Entity);
                    }
                }
                else
                {
                    DataFields.wearingGliderT1 = false;
                }
            }

            // Array colors for swap
            string[] colorArray = new string[11];

            colorArray[0] = "blue";
            colorArray[1] = "red";
            colorArray[2] = "green";
            colorArray[3] = "yellow";
            colorArray[4] = "plain";
            colorArray[5] = "black";
            colorArray[6] = "brown";
            colorArray[7] = "pink";
            colorArray[8] = "purple";
            colorArray[9] = "orange";
            colorArray[10] = "rainbow";

            // Swap Glider folded / unfolded
            foreach (var slot in gearInv)
            {
                ItemWearable item;
                if (slot.Empty || (item = slot.Itemstack.Collectible as ItemWearable) == null) continue; // null Exception catch

                foreach (string color in colorArray)
                {
                    if (DataFields.inGliderMode && slot.Itemstack.Collectible.Code.Path.StartsWith("glider-body-crafted-folded-" + color))
                    {
                        Item gliderUnfolded = api.World.GetItem(new AssetLocation("vsmovemod:glider-body-crafted-unfolded-" + color));
                        ItemStack stackGliderUnfolded = new ItemStack(gliderUnfolded, 1);

                        slot.OnItemSlotModified(slot.Itemstack);
                        {
                            Item gliderFolded = api.World.GetItem(new AssetLocation("vsmovemod:glider-body-crafted-folded-" + color));
                            ItemStack stackGliderFolded = new ItemStack(gliderFolded, 1);

                            slot.Itemstack = stackGliderFolded;
                        }

                        slot.Itemstack = stackGliderUnfolded;
                        slot.MarkDirty();

                        api.World.PlaySoundFor(new AssetLocation("sounds/wearable/brigandine1"), player, false, 100, 5);
                    }
                    else if (!DataFields.inGliderMode && DataFields.timeLanded == 0 && slot.Itemstack.Collectible.Code.Path.StartsWith("glider-body-crafted-unfolded-" + color))
                    {
                        Item gliderFolded = api.World.GetItem(new AssetLocation("vsmovemod:glider-body-crafted-folded-" + color));
                        ItemStack stackGliderFolded = new ItemStack(gliderFolded, 1);

                        slot.OnItemSlotModified(slot.Itemstack);
                        {
                            slot.Itemstack = stackGliderFolded;
                        }

                        slot.Itemstack = stackGliderFolded;
                        slot.MarkDirty();

                        api.World.PlaySoundFor(new AssetLocation("sounds/wearable/brigandine2"), player, false, 100, 5);
                    }
                }
            }

            // last X 
            DataFields.refresh_lastX++;
            if (DataFields.refresh_lastX == 10) // Refresh E_pot every 100 x 10ms = 1s
            {
                if (!DataFields.inert)
                {
                    DataFields.lastX = player.Entity.Pos.Motion.X;
                }
                DataFields.refresh_lastX = 0;
            }

            // last Z
            DataFields.refresh_lastZ++;
            if (DataFields.refresh_lastZ == 10) // Refresh E_pot every 100 x 10ms = 1s
            {
                if (!DataFields.inert)
                {
                    DataFields.lastZ = player.Entity.Pos.Motion.Z;
                }
                DataFields.refresh_lastZ = 0;
            }

            // Get Windspeed at playerPos
            DataFields.windSpeed = player.Entity.World.BlockAccessor.GetWindSpeedAt(player.Entity.Pos.XYZ).X;

            // Get Temperatur at playerPos
            ClimateCondition getCLimateatPlayer = player.Entity.World.BlockAccessor.GetClimateAt(player.Entity.Pos.AsBlockPos);
            DataFields.temperature = getCLimateatPlayer.Temperature;

            // Movement (=MotionSumXZ)
            if (DataFields.motionSumXZ > 0)
            {
                DataFields.motionSumXZ = (Math.Abs(player.Entity.Pos.Motion.X) + Math.Abs(player.Entity.Pos.Motion.Z));
            }
            else // prevent 0 to prevent unlimited fallspeed
            {
                DataFields.motionSumXZ = (Math.Abs(player.Entity.Pos.Motion.X) + Math.Abs(player.Entity.Pos.Motion.Z)) + 0.001;
            }

            // E_pot (Value-Abs.|Y|)
            DataFields.refreshE_pot++;

            if (DataFields.refreshE_pot == 50) // Refresh E_pot every 100 x 10ms = 1s
            {
                DataFields.refreshE_pot = 0;

                if (player.Entity.Pos.Motion.Y < 0)
                {
                    double calcValueAbsYplus1 = 1 + (Math.Abs(player.Entity.Pos.Motion.Y)); // 1 - unlimited
                    DataFields.E_pot = calcValueAbsYplus1;
                }
                else if (player.Entity.Pos.Motion.Y >= 0)
                {
                    DataFields.E_pot = 1;
                }
            }

            //F_drag = motionSumXZ * modWindSpeed * Form_modTemperature (1.004 for 40C to 1.09 for -20 C ~ 9 %) * surface_drag;
            DataFields.F_drag = (DataFields.motionSumXZ * 0.6) * DataFields.modWindSpeed * (2.09 - Math.Pow(DataFields.modTemperature, (1.0 / 2.0))) * DataFields.surface_drag;

            if (DataFields.F_drag >= DataFields.motionSumXZ)
            {
                DataFields.flightIsStalling = true;
            }
            else
            {
                DataFields.flightIsStalling = false;
            }

            //F_lift = (motionSumXZ * modWindSpeed * modTemperature * surface_lift) + (E_pot * 0.015) => (min 1 * 0.015 => min F_lift == 0.015 for motionXZ = 0);
            DataFields.F_lift = (DataFields.motionSumXZ * DataFields.modWindSpeed * DataFields.modTemperature * DataFields.surface_lift) + (DataFields.E_pot * 0.015);

            if (DataFields.F_lift > DataFields.F_gravity)
            {
                DataFields.liftGreaterGravity = true;
            }
            else
            {
                DataFields.liftGreaterGravity = false;
            }

            // Temperature-Modificator
            DataFields.modTemperature = Math.Pow((DataFields.temperature + 21.0), (1.0 / 25.0)); // 1.0 to 1.17

            // Windspeed-Modificator
            if (DataFields.windSpeed >= 0.1)
            {
                DataFields.modWindSpeed = Math.Pow(DataFields.windSpeed, (1.0 / 3.0)); // 0.46 to 1.14
            }
            else
            {
                DataFields.modWindSpeed = Math.Pow(0.1, (1.0 / 3.0)); // min windspeed 0.1 => min modWindSpeed 0.46
            }

            //  ************* Activate Glider-mode  *************
            if (player.Entity.CollidedVertically && !player.Entity.Controls.IsClimbing && !DataFields.isHanging && DataFields.toggleGlider && !player.Entity.Controls.Jump && DataFields.stamina > 0 && player.InventoryManager.ActiveHotbarSlot.Empty && DataFields.wearingGliderT1)
            {
                DataFields.inGliderMode = true;
                player.Entity.StartAnimation("readyglider");
            }

            // Glide when in air & Glider-mode on
            Block air = api.World.GetBlock(new AssetLocation("air"));
            Block below2Blocks = api.World.BlockAccessor.GetBlock((int)player.Entity.Pos.X, (int)player.Entity.Pos.Y - 2, (int)player.Entity.Pos.Z);
            bool below2IsAir = (below2Blocks == air);

            if (DataFields.inGliderMode && !player.Entity.CollidedVertically && !DataFields.playerIsClimbing && !DataFields.isHanging && DataFields.stamina > 0 && below2IsAir && player.InventoryManager.ActiveHotbarSlot.Empty && DataFields.wearingGliderT1)
            {
                if (DataFields.gliderDelay < 200) // Delay_Timer
                {
                    DataFields.gliderDelay++;
                }

                // Enable glider after min. Delay-elapsed
                if (DataFields.gliderDelay > 10)
                {
                    // Set Properties
                    DataFields.hasGlided = true;
                    DataFields.stamina -= DataFields.drainStaminaGliding;
                    player.Entity.Properties.SetEyeHeight(0.6);
                    api.Input.InWorldAction += MoveMod.Input_InWorldAction4;
                    player.Entity.StopAnimation("readyglider");
                    Particles.SpawnFlightParticles(player.Entity);

                    // ************* Control player motion XZ  *************
                    Vec3d walkVector = player.Entity.Controls.WalkVector * 0.3;  // Input speed modifier = increase for "powered version"

                    if (!player.Entity.Controls.Forward && !player.Entity.Controls.Backward && !player.Entity.Controls.Left && !player.Entity.Controls.Right && !player.Entity.Controls.Sprint && !player.Entity.Controls.Jump && DataFields.gliderInertTimer < 50)
                    {
                        DataFields.gliderInertTimer++;
                        double mod = 0.997;

                        player.Entity.Pos.Motion.X = DataFields.lastX * mod;
                        DataFields.lastX = DataFields.lastX * mod;
                        player.Entity.Pos.Motion.Z = DataFields.lastZ * mod;
                        DataFields.lastZ = DataFields.lastZ * mod;
                    }
                    else
                    {
                        player.Entity.Pos.Motion.Add(walkVector * Math.Pow(DataFields.E_pot, 3));
                        player.Entity.Pos.Motion.Sub(walkVector * DataFields.F_drag);

                        DataFields.gliderInertTimer = 0;
                    }

                    // Set bool inert
                    if (!(DataFields.gliderInertTimer == 0) && !(DataFields.gliderInertTimer == 50))
                    {
                        DataFields.inert = true;
                    }
                    else
                    {
                        DataFields.inert = false;
                    }

                    // Fallspeed (dY)
                    // Lift < Gravity
                    double var = 0.02; // modifier lift to deltaY

                    if (DataFields.gliderDelay > 20)
                    {
                        if (!DataFields.liftGreaterGravity)
                        {
                            player.Entity.Pos.Motion.Y = (-1 / DataFields.F_lift) * var;
                        }
                        // Lift > Gravity
                        else
                        {
                            player.Entity.Pos.Motion.Y = (DataFields.F_lift) * (var * 1);
                        }
                    }

                    // Increase Lift_surface (Liftup)
                    if (player.Entity.Controls.Jump && DataFields.gliderDelay == 200)

                    {
                        DataFields.stamina -= DataFields.drainStaminaLift;

                        DataFields.surface_lift = DataFields.surface_liftBase * 3; // 2 -> 3 seems good
                        DataFields.surface_drag = DataFields.surface_dragBase * 3; // 3 -> 3 seems good
                    }
                    else
                    {
                        DataFields.surface_lift = DataFields.surface_liftBase;
                        DataFields.surface_drag = DataFields.surface_dragBase;
                    }

                    // Gliding animations =============================================================================================
                    if (!DataFields.glideAnimToggled)
                    {
                        player.Entity.StartAnimation("useglider");
                        DataFields.glideAnimToggled = true;
                    }

                    bool useGliderLeftToggle = false;
                    bool useGliderRightToggle = false;
                    bool useGliderBackToggle = false;
                    bool useGliderClimbToggle = false;
                    bool useGLiderFallToggle = false;

                    if (player.Entity.Controls.Left && !DataFields.nearGround && DataFields.landingAnimBuffer == 0)
                    {
                        if (!useGliderLeftToggle)
                        {
                            player.Entity.StopAnimation("useglider");
                            player.Entity.StopAnimation("usegliderright");
                            player.Entity.StopAnimation("usegliderback");
                            player.Entity.StopAnimation("usegliderfall");
                            player.Entity.StopAnimation("usegliderclimb");

                            player.Entity.StartAnimation("usegliderleft");

                            useGliderLeftToggle = true;
                        }
                    }
                    else if (player.Entity.Controls.Right && !DataFields.nearGround && DataFields.landingAnimBuffer == 0)
                    {
                        if (!useGliderRightToggle)
                        {
                            player.Entity.StopAnimation("useglider");
                            player.Entity.StopAnimation("usegliderleft");
                            player.Entity.StopAnimation("usegliderback");
                            player.Entity.StopAnimation("usegliderfall");
                            player.Entity.StopAnimation("usegliderclimb");

                            player.Entity.StartAnimation("usegliderright");

                            useGliderRightToggle = true;
                        }
                    }
                    else if (player.Entity.Controls.Left && player.Entity.Controls.Right && !DataFields.nearGround && DataFields.landingAnimBuffer == 0)
                    {
                        if (!useGliderRightToggle && !useGliderRightToggle)
                        {
                            player.Entity.StopAnimation("useglider");
                            player.Entity.StopAnimation("usegliderright");
                            player.Entity.StopAnimation("usegliderleft");
                            player.Entity.StopAnimation("usegliderback");
                            player.Entity.StopAnimation("usegliderclimb");

                            player.Entity.StartAnimation("usegliderfall");

                            useGLiderFallToggle = true;
                        }
                    }
                    else if (player.Entity.Controls.Backward && !DataFields.nearGround && DataFields.landingAnimBuffer == 0)
                    {
                        if (!useGliderBackToggle)
                        {
                            player.Entity.StopAnimation("useglider");
                            player.Entity.StopAnimation("usegliderright");
                            player.Entity.StopAnimation("usegliderleft");
                            player.Entity.StopAnimation("usegliderfall");
                            player.Entity.StopAnimation("usegliderclimb");

                            player.Entity.StartAnimation("usegliderback");

                            useGliderBackToggle = true;
                        }
                    }
                    else if (player.Entity.Controls.Jump && !DataFields.nearGround && DataFields.landingAnimBuffer == 0)
                    {
                        if (!useGliderClimbToggle)
                        {
                            player.Entity.StopAnimation("useglider");
                            player.Entity.StopAnimation("usegliderright");
                            player.Entity.StopAnimation("usegliderleft");
                            player.Entity.StopAnimation("usegliderback");
                            player.Entity.StopAnimation("usegliderfall");

                            player.Entity.StartAnimation("usegliderclimb");

                            useGliderClimbToggle = true;
                        }
                    }
                    else
                    {
                        Block belowBlock7 = api.World.BlockAccessor.GetBlock((int)player.Entity.Pos.X, (int)player.Entity.Pos.Y - 7, (int)player.Entity.Pos.Z);
                        Block belowBlock6 = api.World.BlockAccessor.GetBlock((int)player.Entity.Pos.X, (int)player.Entity.Pos.Y - 6, (int)player.Entity.Pos.Z);
                        Block belowBlock5 = api.World.BlockAccessor.GetBlock((int)player.Entity.Pos.X, (int)player.Entity.Pos.Y - 5, (int)player.Entity.Pos.Z);
                        Block belowBlock4 = api.World.BlockAccessor.GetBlock((int)player.Entity.Pos.X, (int)player.Entity.Pos.Y - 4, (int)player.Entity.Pos.Z);
                        Block belowBlock3 = api.World.BlockAccessor.GetBlock((int)player.Entity.Pos.X, (int)player.Entity.Pos.Y - 3, (int)player.Entity.Pos.Z);
                        Block belowBlock2 = api.World.BlockAccessor.GetBlock((int)player.Entity.Pos.X, (int)player.Entity.Pos.Y - 2, (int)player.Entity.Pos.Z);
                        Block belowBlock1 = api.World.BlockAccessor.GetBlock((int)player.Entity.Pos.X, (int)player.Entity.Pos.Y - 1, (int)player.Entity.Pos.Z);

                        // Spawn particles gliding close over water
                        if (belowBlock1.IsLiquid() || belowBlock2.IsLiquid() || belowBlock3.IsLiquid() || belowBlock4.IsLiquid() || belowBlock5.IsLiquid() || belowBlock6.IsLiquid())
                        {
                            Particles.SpawnWaterSplashParticles(player.Entity);
                            DataFields.playerGlidingOverWater = true;
                        }
                        else
                        {
                            DataFields.playerGlidingOverWater = false;
                        }

                        // Flight Anim
                        if (belowBlock1 == air && belowBlock2 == air && belowBlock3 == air && belowBlock4 == air && belowBlock5 == air && belowBlock6 == air && belowBlock7 == air)
                        {
                            if (!DataFields.inert && DataFields.landingAnimBuffer == 0)
                            {
                                player.Entity.StopAnimation("usegliderleft");
                                player.Entity.StopAnimation("usegliderright");
                                player.Entity.StopAnimation("usegliderback");
                                player.Entity.StopAnimation("usegliderclimb");
                                player.Entity.StopAnimation("readytoland");

                                if (player.Entity.Pos.Motion.Y > -0.1)
                                {
                                    player.Entity.StopAnimation("usegliderfall");
                                    player.Entity.StartAnimation("useglider");

                                    useGliderLeftToggle = false;
                                    useGliderRightToggle = false;
                                    useGliderBackToggle = false;
                                    useGliderClimbToggle = false;
                                    useGLiderFallToggle = false;

                                    DataFields.nearGround = false;
                                }
                                else
                                {
                                    player.Entity.StopAnimation("useglider");
                                    player.Entity.StartAnimation("usegliderfall");

                                    useGliderLeftToggle = false;
                                    useGliderRightToggle = false;
                                    useGliderBackToggle = false;
                                    useGliderClimbToggle = false;
                                    useGLiderFallToggle = false;

                                    DataFields.nearGround = false;
                                }
                            }
                        }
                        else
                        {
                            if (DataFields.gliderDelay == 200) // only play ready to land animation after glider Delay is maxed
                            {
                                player.Entity.StopAnimation("useglider");
                                player.Entity.StopAnimation("usegliderright");
                                player.Entity.StopAnimation("usegliderleft");
                                player.Entity.StopAnimation("usegliderback");
                                player.Entity.StopAnimation("usegliderfall");
                                player.Entity.StopAnimation("usegliderclimb");

                                player.Entity.StartAnimation("readytoland");

                                useGliderLeftToggle = false;
                                useGliderRightToggle = false;
                                useGliderBackToggle = false;
                                useGliderClimbToggle = false;
                                useGLiderFallToggle = false;

                                DataFields.nearGround = true;

                                DataFields.landingAnimBuffer = 50; // hold animation for at leat 50 * 10ms = 0.5s
                            }
                        }
                    }

                    // Animation Buffer Counter
                    if (DataFields.landingAnimBuffer > 0)
                    {
                        DataFields.landingAnimBuffer--;
                    }

                    // Gliding sounds passive
                    if (DataFields.soundGlidingCountdown == 0)
                    {
                        Random rnd = new Random();
                        api.World.PlaySoundFor(new AssetLocation("sounds/wearable/leather1"), player, false, 100, 10);
                        DataFields.soundGlidingCountdown = 25; // Delay Flight Sound
                    }
                    if (DataFields.soundGlidingCountdown < 10 && !DataFields.nearGround)
                    {
                        Random rnd = new Random();
                        api.World.PlaySoundFor(new AssetLocation("sounds/environment/wind"), player, false, 400, 50);
                        DataFields.soundGlidingCountdown = 180;
                    }
                    if (DataFields.soundGlidingCountdown < 180 && DataFields.nearGround)
                    {
                        Random rnd = new Random();
                        api.World.PlaySoundFor(new AssetLocation("sounds/effect/swoosh"), player, false, 100, 10);
                        DataFields.soundGlidingCountdown = rnd.Next(180, 240);
                    }
                    if (DataFields.soundGlidingOverWaterCd < 20 && DataFields.playerGlidingOverWater)
                    {
                        api.World.PlaySoundFor(new AssetLocation("vsmovemod:sounds/waterglide"), player, false, 100, 1000);
                        DataFields.soundGlidingOverWaterCd = 40;
                    }

                    // Gliding sounds on Player-controls
                    if ((player.Entity.Controls.Backward || player.Entity.Controls.Left || player.Entity.Controls.Right || player.Entity.Controls.Jump) && !DataFields.hasPlayedControlSound)
                    {
                        api.World.PlaySoundFor(new AssetLocation("sounds/effect/swoosh"), player, false, 100, 10);
                        DataFields.hasPlayedControlSound = true;
                    }
                    if (!player.Entity.Controls.Backward && !player.Entity.Controls.Left && !player.Entity.Controls.Right && !player.Entity.Controls.Jump)
                    {
                        DataFields.hasPlayedControlSound = false;
                    }
                }
            }

            // Cancel Glider-mode
            if ((DataFields.inGliderMode && (DataFields.stamina < 1 || !DataFields.toggleGlider || !DataFields.wearingGliderT1 || player.Entity.FeetInLiquid || player.Entity.Controls.IsClimbing || (DataFields.hasGlided && player.Entity.CollidedVertically))) || (!DataFields.hasGlided && player.Entity.Controls.Jump) || (DataFields.inGliderMode && player.Entity.Controls.FloorSitting) || (DataFields.inGliderMode && player.Entity.Controls.Sneak) || (DataFields.inGliderMode && DataFields.isCrawling) || (DataFields.inGliderMode && !player.InventoryManager.ActiveHotbarSlot.Empty))
            {

                api.Input.InWorldAction += MoveMod.Input_InWorldAction1;

                // Land on ground sound
                if (DataFields.hasGlided && player.Entity.CollidedVertically)
                {
                    api.World.PlaySoundFor(new AssetLocation("sounds/block/gravel"), player, false, 100, 10);
                }

                if (DataFields.hasGlided)
                {
                    DataFields.timeLanded = 30; // Delay for folding wings
                }

                DataFields.toggleGlider = false;
                DataFields.inGliderMode = false;
                DataFields.hasGlided = false;
                DataFields.gliderDelay = 0;
                DataFields.glideAnimToggled = false;

                player.Entity.StopAnimation("useglider");
                player.Entity.StopAnimation("usegliderright");
                player.Entity.StopAnimation("usegliderleft");
                player.Entity.StopAnimation("usegliderback");
                player.Entity.StopAnimation("usegliderfall");
                player.Entity.StopAnimation("usegliderclimb");
                player.Entity.StopAnimation("readyglider");
                player.Entity.StopAnimation("readytoland");

                player.Entity.Properties.SetEyeHeight(1.7);
                api.Input.InWorldAction += MoveMod.Input_InWorldAction1;
            }

            if (!DataFields.hasGlided && DataFields.timeLanded > 0)
            {
                DataFields.timeLanded--;
            }
        }
    }
}
