﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Vintagestory.API.Client;
using Vintagestory.API.Server;
using Vintagestory.API.Common;
using Vintagestory.GameContent;
using Vintagestory.API.Config;
using Vintagestory.API.MathTools;
using Vintagestory.API.Common.Entities;
using ProtoBuf;

namespace vsmovemod.src
{
    class Particles
    {
        public static void SpawnFlightParticles(Entity byEntity)
        {
            DataFields.myParticles.MinPos = byEntity.Pos.XYZ.Add(0, (byEntity.Properties.EyeHeight - 1.5), 0).Ahead(-1.5f, byEntity.Pos.Pitch, byEntity.Pos.Yaw);
            DataFields.myParticles.AddPos = new Vec3d(0.5, 0.5, 0.5);
            DataFields.myParticles.LifeLength = 1F;
            DataFields.myParticles.GravityEffect = 0.1F;
            DataFields.myParticles.SizeEvolve = new EvolvingNatFloat(EnumTransformFunction.LINEAR, 4);
            DataFields.myParticles.ParticleModel = EnumParticleModel.Quad;
            DataFields.myParticles.Color = ColorUtil.ColorFromRgba(255, 255, 255, 30);
            DataFields.myParticles.OpacityEvolve = new EvolvingNatFloat(EnumTransformFunction.LINEAR, -150);
            byEntity.World.SpawnParticles(DataFields.myParticles);
        }

        public static void SpawnWaterParticles(Entity byEntity, float bodyYaw, float zValue)
        {
            DataFields.myWindParticles.MinPos = byEntity.Pos.XYZ.Add(0, (byEntity.Properties.EyeHeight - 1), 0).Ahead(-1.8f, 0, bodyYaw + zValue);
            DataFields.myWindParticles.AddPos = new Vec3d(0, 0, 1);
            DataFields.myWaterSplashParticles.MaxSize = 0.001F;
            DataFields.myWaterSplashParticles.MinQuantity = 1.5f;
            DataFields.myWindParticles.LifeLength = 1F;
            DataFields.myWindParticles.GravityEffect = 0.01F;
            DataFields.myWindParticles.SizeEvolve = new EvolvingNatFloat(EnumTransformFunction.LINEAR, 0.2f);
            DataFields.myWindParticles.ParticleModel = EnumParticleModel.Cube;
            DataFields.myWindParticles.Color = ColorUtil.ColorFromRgba(245, 233, 154, 0);
            DataFields.myWindParticles.OpacityEvolve = new EvolvingNatFloat(EnumTransformFunction.LINEAR, -2);
            DataFields.myWaterSplashParticles.ShouldDieInLiquid = true;
            byEntity.World.SpawnParticles(DataFields.myWindParticles);
        }

        public static void SpawnSpecialParticles(Entity byEntity)
        {
            DataFields.mySpecialParticles.MinPos = byEntity.Pos.XYZ.Add(0, 0.5, -0.5).Ahead(-0.5f, byEntity.Pos.Pitch, byEntity.Pos.Yaw);
            DataFields.mySpecialParticles.AddPos = new Vec3d(1, 0.6, 1.2);
            DataFields.mySpecialParticles.MaxSize = 0.001F;
            DataFields.mySpecialParticles.LifeLength = 0.3F;
            DataFields.mySpecialParticles.GravityEffect = -0.01F;
            DataFields.mySpecialParticles.ParticleModel = EnumParticleModel.Cube;
            DataFields.mySpecialParticles.MinQuantity = 0.2f;
            Random rand = new Random();
            DataFields.mySpecialParticles.Color = ColorUtil.ColorFromRgba(255, rand.Next(0, 255), rand.Next(0, 255), rand.Next(0, 255));
            DataFields.mySpecialParticles.OpacityEvolve = new EvolvingNatFloat(EnumTransformFunction.LINEAR, -255);
            byEntity.World.SpawnParticles(DataFields.mySpecialParticles);
        }

        public static void SpawnWaterSplashParticles(Entity byEntity)
        {
            DataFields.myWaterSplashParticles.MinPos = byEntity.Pos.XYZ.Add(0, -0.3, 0).Ahead(-0.7f, byEntity.Pos.Pitch, byEntity.Pos.Yaw);
            DataFields.myWaterSplashParticles.AddPos = new Vec3d(0, 0.1, 1.2);
            DataFields.myWaterSplashParticles.MaxSize = 0.002F;
            DataFields.myWaterSplashParticles.LifeLength = 1F;
            DataFields.myWaterSplashParticles.GravityEffect = 0.01F;
            DataFields.myWaterSplashParticles.ShouldDieInLiquid = true;
            DataFields.myWaterSplashParticles.ParticleModel = EnumParticleModel.Cube;
            DataFields.myWaterSplashParticles.MinQuantity = 1.5f;
            DataFields.myWaterSplashParticles.Color = ColorUtil.ColorFromRgba(255, 255, 256, 30);
            DataFields.myWaterSplashParticles.OpacityEvolve = new EvolvingNatFloat(EnumTransformFunction.LINEAR, -255);
            byEntity.World.SpawnParticles(DataFields.myWaterSplashParticles);
        }
    }
}
